/*
 *  By: Sunil Kumar
 *  Cad2D Module. Module to draw simple 2D geometry objects and serialization using GeoIO module.
 *  This code is free and is distributed in the hope that it will be useful for simple learning and experimental purpose only WITHOUT any WARRANTY.
 *  The author shall hold no redistribution liability in case of any unethical usage/modification.
 *  Distributed under CPOL.
 */
// CadView.cpp : implementation of the CCadView class
//

#include "stdafx.h"
// SHARED_HANDLERS can be defined in an ATL project implementing preview, thumbnail
// and search filter handlers and allows sharing of document code with that project.
#ifndef SHARED_HANDLERS
#include "Cad2D.h"
#endif

#include "CadDoc.h"
#include "CadView.h"
#include "PolgonDataDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CCadView

IMPLEMENT_DYNCREATE(CCadView, CView)

BEGIN_MESSAGE_MAP(CCadView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CView::OnFilePrintPreview)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_DRAW_LINE, &CCadView::OnDrawLine)
	ON_COMMAND(ID_DRAW_CIRCLE, &CCadView::OnDrawCircle)
	ON_COMMAND(ID_DRAW_RECTANGLE, &CCadView::OnDrawRectangle)
	ON_COMMAND(ID_DRAW_POLYGON, &CCadView::OnDrawPolygon)
	ON_COMMAND(ID_DRAW_CANCEL, &CCadView::OnDrawCancel)
END_MESSAGE_MAP()

// CCadView construction/destruction

CCadView::CCadView()
{
	m_nNumOfSidesOfPolygon = 0;
	m_nNumOfInputs = 0;
	m_nCurrOperation = 0;
	m_nCurrEntity = GEOIO_UNDEFINED;
}

CCadView::~CCadView()
{
}

BOOL CCadView::PreCreateWindow(CREATESTRUCT& cs)
{
	HBRUSH   hblack = ::CreateSolidBrush(RGB(0,0,0));
	HCURSOR  hcursor = 	AfxGetApp()->LoadStandardCursor(IDC_CROSS);

	// make the default white window to black
	try
	{
		cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW , hcursor, hblack);
	}
	catch(.../*CResourceException ex*/)
	{
		/*ex.ReportError();*/
	}

	return CView::PreCreateWindow(cs);
}

// CCadView drawing

void CCadView::OnDraw(CDC* pDC)
{
	CCadDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	pDoc->RefreshView(pDC);
}


// CCadView printing

BOOL CCadView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CCadView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CCadView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CCadView diagnostics

#ifdef _DEBUG
void CCadView::AssertValid() const
{
	CView::AssertValid();
}

void CCadView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CCadDoc* CCadView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCadDoc)));
	return (CCadDoc*)m_pDocument;
}
#endif //_DEBUG


// CCadView message handlers

void CCadView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	m_BlackPen.CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	m_WhitePen.CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
	m_pMainFrm = (CMainFrame*)AfxGetMainWnd();
}

void CCadView::OnMouseMove(UINT nFlags, CPoint point)
{
	m_pntCursor.x = point.x;
	m_pntCursor.y = point.y;
	if(m_nCurrOperation != 0)
	{
		PerformDragging(point);
	}
	m_pntPrevForDrag = point;
		

	// TODO: Add your message handler code here and/or call default
	CString strCoordinates(_T(""));
	strCoordinates.Format(_T("X: %d"), point.x);
	m_pMainFrm->m_wndStatusBar.SetPaneText(1, strCoordinates, TRUE);
	strCoordinates.Format(_T("Y: %d"), point.y);
	m_pMainFrm->m_wndStatusBar.SetPaneText(2, strCoordinates, TRUE);	

	CView::OnMouseMove(nFlags, point);
}


void CCadView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// TODO: Add your message handler code here and/or call default
	ProcessInputs(point);

	CView::OnLButtonDown(nFlags, point);
}

void CCadView::ResetInputs()
{
	m_nNumOfSidesOfPolygon = 0;
	m_nNumOfInputs = 0;
	m_nCurrOperation = 0;
	m_nCurrEntity = GEOIO_UNDEFINED;

	this->Invalidate(); // causes a redraw
}

void CCadView::OnDrawLine()
{
	m_nNumOfInputs = 0;
	m_nCurrOperation = DRAW_LINE;
	ProcessLineInput(m_pntCursor);
}


void CCadView::OnDrawCircle()
{
	m_nNumOfInputs = 0;
	m_nCurrOperation = DRAW_CIRCLE;
	ProcessCircleInput(m_pntCursor);
}


void CCadView::OnDrawRectangle()
{
	m_nNumOfInputs = 0;
	m_nCurrOperation = DRAW_RECTANGLE;
	ProcessRectangleInput(m_pntCursor);
}


void CCadView::OnDrawPolygon()
{
	m_nNumOfInputs = 0;
	m_nCurrOperation = DRAW_POLYGON;
	ProcessPolygonInput(m_pntCursor);
}

void CCadView::ProcessInputs(CPoint pnt)
{
	switch(m_nCurrOperation)
	{
		case DRAW_LINE: ProcessLineInput(pnt);
			break;	

		case DRAW_CIRCLE: ProcessCircleInput(pnt);
			break;	

		case DRAW_RECTANGLE: ProcessRectangleInput(pnt);
			break;

		case DRAW_POLYGON: ProcessPolygonInput(pnt);
			break;	

		default:
			break;
	}

	// Notify all the views to refresh
	CCadDoc *pDoc = GetDocument();
	pDoc->UpdateAllViews(NULL, 0L, NULL);
}

void CCadView::ProcessLineInput(CPoint pnt)
{
	if( m_nNumOfInputs == 0)
	{
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the first point of the line"), TRUE);
		m_nNumOfInputs++;
	}
	else if( m_nNumOfInputs == 1)
	{
		m_pntFirst = pnt;
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the end point of the line"), TRUE);
		m_nNumOfInputs++;
	}
	else if( m_nNumOfInputs == 2)
	{
		m_pntSecond = pnt;
		CGeoLine *pEntity = new CGeoLine();
		pEntity->m_StPnt = m_pntFirst;
		pEntity->m_EndPnt = m_pntSecond;
		CDC *pDC = GetDC();
		OnPrepareDC(pDC, NULL);
		pEntity->m_clr = RGB(255, 255, 255);
		pEntity->Draw(pDC);

		CCadDoc *pDoc = GetDocument();
		pDoc->AddEntity(pEntity);
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T(""), TRUE);		
		this->ResetInputs();
	}
}

void CCadView::ProcessCircleInput(CPoint pnt)
{
	if( m_nNumOfInputs == 0)
	{
		m_nNumOfInputs++;
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the centre point of the circle"), TRUE);		
	}
	else if( m_nNumOfInputs == 1)
	{
		m_pntFirst = pnt;
		m_nNumOfInputs++;
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the radial point of the circle"), TRUE);
	}
	else if( m_nNumOfInputs == 2)
	{
		m_pntSecond = pnt;
		CGeoCircle *pEntity = new CGeoCircle();
		pEntity->m_CentrePnt = m_pntFirst;
		pEntity->m_EndPnt = m_pntSecond;
		CDC *pDC = GetDC();
		OnPrepareDC(pDC, NULL);
		pEntity->m_clr = RGB(255, 255, 255);
		pEntity->Draw(pDC);
	
		CCadDoc *pDoc = GetDocument();
		pDoc->AddEntity(pEntity);
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T(""), TRUE);		
		this->ResetInputs();
	}
}

void CCadView::ProcessRectangleInput(CPoint pnt)
{
	if( m_nNumOfInputs == 0)
	{
	   m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the first corner of the rectangle"), TRUE);
		m_nNumOfInputs++;
	}
	else if( m_nNumOfInputs == 1)
	{
		m_pntFirst = pnt;
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the opposite corner of the rectangle"), TRUE);
		m_nNumOfInputs++;
	}
	else if( m_nNumOfInputs == 2)
	{
		m_pntSecond = pnt;
		CGeoRectangle *pRectangle = new CGeoRectangle;
		pRectangle->m_StPnt = m_pntFirst;
		pRectangle->m_EndPnt = m_pntSecond;
		CDC *pDC = GetDC();
		OnPrepareDC(pDC, NULL);
		pRectangle->m_clr = RGB(255, 255, 255);
		pRectangle->Draw(pDC);
	
		CCadDoc *pDoc = GetDocument();
		pDoc->AddEntity(pRectangle);
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T(""), TRUE);		
		this->ResetInputs();
	}
}

void CCadView::ProcessPolygonInput(CPoint pnt)
{
	if( m_nNumOfInputs == 0)
	{
		m_pntFirst = pnt;
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the centre point of the polygon"), TRUE);

		CPolgonDataDlg dlg;
		if( dlg.DoModal() == IDOK )
		{
			m_nNumOfSidesOfPolygon = dlg.m_nNumOfSides;
			m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the centre point of the polygon"), TRUE);
		}
		else
		{
			m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T(""), TRUE);
			this->ResetInputs();
			return;
		}
		m_nNumOfInputs++;
	}
	else if( m_nNumOfInputs == 1)
	{
		m_pntFirst = pnt;
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T("Click the vertex of side of the polygon"), TRUE);
		m_nNumOfInputs++;
	}
	else if( m_nNumOfInputs == 2)
	{
		m_pntSecond = pnt;
		CGeoPolygon *pPolygon = new CGeoPolygon();
		pPolygon->m_StPnt = m_pntFirst;
		pPolygon->m_EndPnt = m_pntSecond;
		pPolygon->m_nNumOfSides = m_nNumOfSidesOfPolygon;
		m_nNumOfSidesOfPolygon = 0;
		CDC *pDC = GetDC();
		OnPrepareDC(pDC, NULL);
		pPolygon->m_clr = RGB(255, 255, 255);;
		pPolygon->Draw(pDC);
	
		CCadDoc *pDoc = GetDocument();
		pDoc->AddEntity(pPolygon);
		m_pMainFrm->m_wndStatusBar.SetPaneText(0, _T(""), TRUE);		
		this->ResetInputs();
	}
}

void CCadView::OnDrawCancel()
{
	this->ResetInputs();
}

void CCadView::PerformDragging(CPoint pnt)
{
	if( m_nNumOfInputs < 2)
		return;

	switch( m_nCurrOperation )
	{
		case DRAW_LINE:
			{
				CDC *pDC = GetDC();
				OnPrepareDC(pDC, NULL);
				pDC->SelectObject(&m_BlackPen);
				pDC->MoveTo(m_pntFirst.x, m_pntFirst.y);
				pDC->LineTo(m_pntPrevForDrag.x, m_pntPrevForDrag.y);
				pDC->SelectObject(&m_WhitePen);
				pDC->MoveTo(m_pntFirst.x, m_pntFirst.y);
				pDC->LineTo(pnt.x, pnt.y);
				RefreshView();
				break;
			}

		case DRAW_RECTANGLE:
			{
				CDC *pDC = GetDC();
				OnPrepareDC(pDC, NULL);
				pDC->SelectObject(&m_BlackPen);
				pDC->MoveTo(m_pntFirst.x, m_pntFirst.y);
				pDC->LineTo(m_pntPrevForDrag.x, m_pntFirst.y);
				pDC->LineTo(m_pntPrevForDrag.x, m_pntPrevForDrag.y);
				pDC->LineTo(m_pntFirst.x, m_pntPrevForDrag.y);
				pDC->LineTo(m_pntFirst.x, m_pntFirst.y);

				pDC->SelectObject(&m_WhitePen);
				pDC->MoveTo(m_pntFirst.x, m_pntFirst.y);
				pDC->LineTo(pnt.x, m_pntFirst.y);
				pDC->LineTo(pnt.x, pnt.y);
				pDC->LineTo(m_pntFirst.x, pnt.y);
				pDC->LineTo(m_pntFirst.x, m_pntFirst.y);
				RefreshView();
				break;
			}

		case DRAW_CIRCLE:
			{
				CDC *pDC = GetDC();
				OnPrepareDC(pDC, NULL);
				CGeoCircle *pTemp = new CGeoCircle;
				pTemp->m_CentrePnt = m_pntFirst;
				pTemp->m_EndPnt = m_pntPrevForDrag;
				pTemp->DrawCircleForDragging(pDC, RGB(0,0,0));
				delete pTemp;
				pTemp = NULL;

				pTemp = new CGeoCircle;
				pTemp->m_CentrePnt = m_pntFirst;
				pTemp->m_EndPnt = pnt;
				pTemp->m_clr = RGB(255, 255, 255);
				pTemp->Draw(pDC);
				RefreshView();
				delete pTemp;
				pTemp = NULL;
				break;
			}

		case DRAW_POLYGON:
			{
				CDC *pDC = GetDC();
				OnPrepareDC(pDC, NULL);
				m_pntSecond = pnt;
				CGeoPolygon *pTemp = new CGeoPolygon();
				pTemp->m_StPnt = m_pntFirst;
				pTemp->m_EndPnt = m_pntPrevForDrag;
				pTemp->m_nNumOfSides = m_nNumOfSidesOfPolygon;								
				pTemp->Draw(pDC, RGB(0,0,0));
				delete pTemp;
				pTemp = NULL;

				pTemp = new CGeoPolygon();
				pTemp->m_StPnt = m_pntFirst;
				pTemp->m_EndPnt = pnt;
				pTemp->m_nNumOfSides = m_nNumOfSidesOfPolygon;								
				pTemp->Draw(pDC);
				RefreshView();
				delete pTemp;
				pTemp = NULL;
				break;
			}
	}
}

void CCadView::RefreshView()
{
	CDC *pDC = GetDC();
	OnPrepareDC(pDC, NULL);
	GetDocument()->RefreshView(pDC);
}
