/*
 *  By: Sunil Kumar
 *  Cad2D Module. Module to draw simple 2D geometry objects and serialization using GeoIO module.
 *  This code is free and is distributed in the hope that it will be useful for simple learning and experimental purpose only WITHOUT any WARRANTY.
 *  The author shall hold no redistribution liability in case of any unethical usage/modification.
 *  Distributed under CPOL.
 */
// CadView.h : interface of the CCadView class
//

#pragma once
#include "stdafx.h"
#include "MainFrm.h"


#define  DRAW_LINE              1
#define  DRAW_CIRCLE            2
#define  DRAW_RECTANGLE         3
#define  DRAW_POLYGON           4

class CCadView : public CView
{
protected: // create from serialization only
	CCadView();
	DECLARE_DYNCREATE(CCadView)

// Attributes
public:
	CCadDoc* GetDocument() const;

// Operations
public:
	unsigned int m_nNumOfInputs;

// Overrides
public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnInitialUpdate();

// Implementation
public:
	virtual ~CCadView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	CMainFrame* m_pMainFrm;	

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnDrawLine();
	afx_msg void OnDrawCircle();
	afx_msg void OnDrawRectangle();
	afx_msg void OnDrawPolygon();
	afx_msg void OnDrawCancel();

private:
	CPen m_BlackPen;
	CPen m_WhitePen;
	CPoint m_pntPrevForDrag;
	int m_nNumOfSidesOfPolygon;
	CPoint m_pntFirst;
	CPoint m_pntSecond;
	CPoint m_pntCursor;
	GEO_ENTITY_TYPE m_nCurrEntity;
	unsigned int m_nCurrOperation;
	
	void ResetInputs();
	void ProcessInputs(CPoint pnt);
	void ProcessLineInput(CPoint pnt);
	void ProcessCircleInput(CPoint pnt);
	void ProcessRectangleInput(CPoint pnt);
	void ProcessPolygonInput(CPoint pnt);
	void PerformDragging(CPoint pnt);
	void RefreshView();
	
};

#ifndef _DEBUG  // debug version in CadView.cpp
inline CCadDoc* CCadView::GetDocument() const
   { return reinterpret_cast<CCadDoc*>(m_pDocument); }
#endif

