// EntityList.cpp: implementation of the CEntityList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EntityList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEntityList::CEntityList()
{

}

CEntityList::~CEntityList()
{
	// delete all the entities in the entity list //
	while( !m_entitylist.IsEmpty() )
	{
		CPrimitive *pEntity = m_entitylist.GetHead();
		m_entitylist.RemoveHead();
		delete pEntity;
		pEntity = NULL;
	}
}

void CEntityList::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// remove all the deleted entities here //
		CObList *pList = GetEntityList();	
		POSITION pos = pList->GetHeadPosition();
		POSITION prevPos = pos;

		while( pos != NULL )
		{
			CPrimitive *pEntity = (CPrimitive *)pList->GetNext(pos);

			if( pEntity->IsEntityDeleted() == TRUE )
				pList->RemoveAt(prevPos);

			prevPos = pos;

		}
	}
	else
	{
		// TODO: add loading code here
	}
	m_entitylist.Serialize(ar);
}

CObList * CEntityList::GetEntityList()
{
	return &(m_entitylist);
}



/**********************************************************************************
 this function draws the entities stored in the list
**********************************************************************************/
void CEntityList::DrawEntityList(CDC *pDC)
{
	CObList *pList = GetEntityList();	
	if( pList->GetCount() == 0)
		return;

	CPrimitive *pEntity = NULL;
	POSITION pos = pList->GetHeadPosition();
	while( pos )
	{	
		CPrimitive *pEntity  =  (CPrimitive *)pList->GetNext(pos);
		ASSERT(pEntity);
		if( pEntity->IsEntityDeleted() == FALSE )
			pEntity->Draw(pDC);		
	}	
}

/**********************************************************************************
 this function draws the entities stored in the list,  with the color specified
**********************************************************************************/
void CEntityList::DrawEntityList(CDC *pDC, COLORREF clr)
{
	CObList *pList = GetEntityList();	
	if( pList->GetCount() == 0)
		return;

	CPrimitive *pEntity = NULL;
	POSITION pos = pList->GetHeadPosition();
	while( pos )
	{	
		CPrimitive *pEntity  =  (CPrimitive *)pList->GetNext(pos);
		ASSERT(pEntity);
		if( pEntity->IsEntityDeleted() == FALSE )
		{
			pEntity->SetEntityColor(clr);
			pEntity->Draw(pDC);		
		}
	}	
}

/**********************************************************************************
 this function is used to delete all the entities in the list
**********************************************************************************/
void CEntityList::DeleteAllEntities()
{
	if( m_entitylist.GetCount() > 0)
	while( m_entitylist.IsEmpty() == FALSE )
		delete m_entitylist.RemoveHead();
}



/**********************************************************************************
 add entities to list by calling this function
**********************************************************************************/
void CEntityList::AddEntity(CPrimitive *pPrimitive)
{
	m_entitylist.AddTail(pPrimitive);
}


/**********************************************************************************
  Function to get the entity count present in the entity list
**********************************************************************************/
int CEntityList::GetEntityCount()
{
	return ( m_entitylist.GetCount() ) ;
}

/****************************************************************************
	This function gets the last entity in the entity list
****************************************************************************/
CPrimitive* CEntityList::GetLastEntity()
{
	POSITION pos = m_entitylist.GetHeadPosition();
	if( pos == NULL )
		return NULL;

	CPrimitive *pEntity = NULL ;

	while( pos != NULL )
	{
		pEntity = (CPrimitive *)m_entitylist.GetNext(pos);
		ASSERT(pEntity);
	}

	return pEntity;
}

/****************************************************************************
	This function deletes the entity whose entity number is passed as
	parameter
****************************************************************************/
void CEntityList::DeleteEntity(int nEntityNum)
{
	POSITION pos = m_entitylist.GetHeadPosition();
	if( pos == NULL )
		return ;

	POSITION prev = NULL;
	CPrimitive *pEntity = NULL ;
	pEntity = (CPrimitive *)m_entitylist.GetAt(pos);
	while( pos )
	{	
		prev = pos;
		pEntity = (CPrimitive *)m_entitylist.GetNext(pos);
		ASSERT(pEntity);

		if( pEntity->GetEntityNumber() == nEntityNum )
		{
			m_entitylist.RemoveAt(prev);
			return;
		}
		
	}
}

/**************************************************************************
	this function changes the entities color in this layer
**************************************************************************/
void CEntityList::ChangeEntitiesColorInList(COLORREF clr)
{
	CObList *pList = GetEntityList();	
	if( pList->GetCount() == 0)
		return;

	CPrimitive *pEntity = NULL;
	POSITION pos = pList->GetHeadPosition();
	while( pos )
	{	
		CPrimitive *pEntity  =  (CPrimitive *)pList->GetNext(pos);
		ASSERT(pEntity);
		if( pEntity->IsEntityDeleted() == FALSE )
			pEntity->SetEntityColor(clr);				
		
	}	
}


/**************************************************************************
	this function erases the entity near the point passed as parmeter
**************************************************************************/
void CEntityList::EraseEntityNearPoint(CPoint pnt)
{
	CObList *pList = GetEntityList();	
	if( pList->GetCount() == 0)
		return;

	CPrimitive *pEntity = NULL;
	POSITION pos = pList->GetHeadPosition();
	while( pos )
	{	
		/*CPrimitive *pEntity  =  (CPrimitive *)pList->GetNext(pos);
		ASSERT(pEntity);
		
		if( pEntity->IsPointNearEntity(pnt) == TRUE )
		{
			int nNum = pEntity->GetEntityNumber();
			DeleteEntity(nNum);
			return;
		}*/
		break;
		
	}
}

/**************************************************************************************
 Function gets the entity at specified index from the list. not zero based
**************************************************************************************/
CPrimitive* CEntityList::GetEntityAtIndex(int nIndex)
{
	if( nIndex == -1 )
		return NULL;

	POSITION pos = m_entitylist.GetHeadPosition();
	if( pos == NULL )
		return NULL;

	CPrimitive *pEntity = NULL ;
	pEntity = (CPrimitive *)m_entitylist.GetAt(pos);
	ASSERT(pEntity);

	int nCount = 0;
	while( pos )
	{
		pEntity = (CPrimitive *)m_entitylist.GetNext(pos);
		if( nCount == nIndex )
			break;
		
		nCount++;		
	}
		

	return pEntity;
}
