// GeoCircle.cpp: implementation of the CGeoCircle class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GeoCircle.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CGeoCircle, CObject, VERSIONABLE_SCHEMA | GEOIO_MODULE_VERSION_NUM)

CGeoCircle::CGeoCircle()
{
	SetEntityType(GEOIO_CIRCLE1);
}

CGeoCircle::CGeoCircle(CGeoCircle& pEntity)
{
	SetEntityType(pEntity.GetEntityType());
	SetEntityNumber(pEntity.GetEntityNumber());
	//SetStartPoint(pEntity.GetStartPoint());
	SetEndPoint(pEntity.GetEndPoint());
	SetEntityColor(pEntity.GetEntityColor());
	m_fRadius = pEntity.m_fRadius;
}

CGeoCircle::~CGeoCircle()
{
	
}


/***********************************************************************************
		This fuction draws the CIRCLE with m_CentrePnt as centre point and
		m_fRadius as radius. pass the DC of the view class
***********************************************************************************/
void CGeoCircle::Draw(CDC *pDC)
{
	CalculateRadiusFromCentreAndEndPoints();
	CPen pen(PS_SOLID, 0, m_clr);
	pDC->SelectObject(&pen);

	CPoint Point;
	Point.x =(m_CentrePnt.x + m_fRadius);
	Point.y =m_CentrePnt.y;  
	pDC->MoveTo(Point);
	
	float fRadian = 0.1 ;
	// for drawing the circle from 0 to 360 degree simple trigonometry ;-)
	while(fRadian <=(2*3.15))
	{
		Point.x = m_CentrePnt.x + m_fRadius * cos(fRadian);
		Point.y = m_CentrePnt.y + m_fRadius * sin(fRadian);
		pDC->LineTo(Point);
		fRadian += 0.1;
	}
}

// store or load
void CGeoCircle::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << m_nEntityType;
		ar << m_nEntityNum;
		ar << m_CentrePnt.x;
		ar << m_CentrePnt.y;
		ar << m_EndPnt.x;
		ar << m_EndPnt.y;
		ar << m_fRadius;
		ar << m_clr;
	}
	else
	{	
		ar >> m_nEntityType;
		ar >> m_nEntityNum;
		ar >> m_CentrePnt.x;
		ar >> m_CentrePnt.y;
		ar >> m_EndPnt.x;
		ar >> m_EndPnt.y;
		ar >> m_fRadius;
		ar >> m_clr;
	}
}

/***********************************************************************************
	This fuction calculates the radius
***********************************************************************************/
void CGeoCircle::CalculateRadiusFromCentreAndEndPoints()
{
	m_fRadius = sqrt(  pow((double)(m_EndPnt.x - m_CentrePnt.x), 2) + pow( (double)(m_EndPnt.y - m_CentrePnt.y), 2) );
}

/***********************************************************************************
		This fuction draws the CIRCLE with m_CentrePnt as centre point and
		m_fRadius as radius. pass the DC of the view class
***********************************************************************************/
void CGeoCircle::DrawCircleForDragging(CDC *pDC, COLORREF clr)
{
	CalculateRadiusFromCentreAndEndPoints();
	CPen pen(PS_SOLID, 0, clr);
	pDC->SelectObject(&pen);

	CPoint Point;
	Point.x =(m_CentrePnt.x + m_fRadius);
	Point.y =m_CentrePnt.y;  
	pDC->MoveTo(Point);
	
	float fRadianCounter = 0.1 ;
	// for drawing the circle from 0 to 360 degree
	while(fRadianCounter <=(2*3.15))
	{
		Point.x = m_CentrePnt.x + m_fRadius * cos(fRadianCounter);
		Point.y = m_CentrePnt.y + m_fRadius * sin(fRadianCounter);
		pDC->LineTo(Point);
		fRadianCounter += 0.1;
	}
}


BOOL CGeoCircle::IsEntityDeleted()
{
	return m_bEntityDeleted;
}

void CGeoCircle::DeleteEntity()
{
	m_bEntityDeleted = TRUE;
}


BOOL CGeoCircle::IsPointNearEntity(CPoint refPnt)
{
	return 1;
}

/*---------------------------------------------------------------------------------
Function to get the radius of the circle
---------------------------------------------------------------------------------*/
float CGeoCircle::GetRadius()
{
	CalculateRadiusFromCentreAndEndPoints();
	return m_fRadius;
}

/*---------------------------------------------------------------------------------
	Function to mirror entity with respect to X-axis
---------------------------------------------------------------------------------*/
void CGeoCircle::MirrorAboutXaxis()
{
	/* To mirror a ciclr about X-axis, negate the centre point Ycoordinate*/
	m_CentrePnt.y = -(m_CentrePnt.y);
	m_EndPnt.y = -(m_EndPnt.y);
}

/*---------------------------------------------------------------------------------
	Function to mirror entity with respect to y-axis
---------------------------------------------------------------------------------*/
void CGeoCircle::MirrorAboutYaxis()
{
	/* To mirror a ciclr about Y-axis, negate the centre point Xcoordinate*/
	m_CentrePnt.x = -(m_CentrePnt.x);
	m_EndPnt.x = -(m_EndPnt.x);
}

/*---------------------------------------------------------------------------------
	Function to clone the entity. This function returns a new pointer to
	cloned entity.
---------------------------------------------------------------------------------*/
CPrimitive* CGeoCircle::CreateClone()
{
	CGeoCircle* pClone = new CGeoCircle(*this);
	return pClone;
}