// GeoPolybezier.cpp: implementation of the CGeoPolybezier class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GeoPolybezier.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CGeoPolybezier, CObject, VERSIONABLE_SCHEMA | GEOIO_MODULE_VERSION_NUM)

CGeoPolybezier::CGeoPolybezier()
{
	SetEntityType(GEOIO_POLYBEZIER);
}

CGeoPolybezier::CGeoPolybezier(CGeoPolybezier& pEntity)
{
	SetEntityType(pEntity.GetEntityType());
	SetEntityNumber(pEntity.GetEntityNumber());
	SetStartPoint(pEntity.GetStartPoint());
	SetEndPoint(pEntity.GetEndPoint());
	SetEntityColor(pEntity.GetEntityColor());
	m_ControlPoints.Copy(pEntity.m_ControlPoints);
}

CGeoPolybezier::~CGeoPolybezier()
{
	m_ControlPoints.RemoveAll();
}


// store or load
void CGeoPolybezier::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{	// storing code
		
	}
	else
	{	// loading code
		
	}

	CPrimitive::Serialize(ar);
	m_ControlPoints.Serialize(ar);
}

/***********************************************************************************
	Function to determine if the point passed is near the entity
***********************************************************************************/
void CGeoPolybezier::PointNearEntity(CPoint pnt)
{
}

/***********************************************************************************
		This fuction draws the rectangle with m_StPnt as top left point and
		m_EndPnt as bottom right point. pass the DC of the view class
***********************************************************************************/
void CGeoPolybezier::Draw(CDC *pDC)
{
	CPen pen(PS_SOLID, 0, m_clr);
	pDC->SelectObject(&pen);

	pDC->MoveTo(m_StPnt);
	if(m_ControlPoints.GetSize()>1)
		pDC->PolyBezierTo( m_ControlPoints.GetData(), m_ControlPoints.GetSize());
	
}

void CGeoPolybezier::AddControlPoint(CPoint pnt)
{
	m_ControlPoints.Add(pnt);
}

void CGeoPolybezier::DeleteEntity()
{
	SetDeleted();
}

/***********************************************************************************
	i dont know
***********************************************************************************/
int CGeoPolybezier::IsInsideControlPoint(POINT& point)
{
	int count = m_ControlPoints.GetSize();

	if(count>0)	
	{
		for(int i=0; i<count; i++)
		{
			if(Distance(m_ControlPoints[i], point)<5)
				return i;
		}
	}

	return -1;

}

/***********************************************************************************
	i dont know
***********************************************************************************/
double CGeoPolybezier::Distance(const POINT& p1, const POINT& p2)
{
	double dx = abs(p1.x - p2.x);
	double dy = abs(p1.y - p2.y);

	return sqrt(dx*dx + dy*dy);
}

BOOL CGeoPolybezier::IsPointNearEntity(CPoint refPnt)
{
	return TRUE;
}

/*---------------------------------------------------------------------------------
	Function to mirror entity with respect to X-axis
---------------------------------------------------------------------------------*/
void CGeoPolybezier::MirrorAboutXaxis()
{
	/* Negate the Ycoordinate of the start and end points, and then the
	   control points */
	m_StPnt.y = -(m_StPnt.y);	
	m_EndPnt.y = -(m_EndPnt.y);

	/*Time to negate the control points' Ycoordinate*/
	if(m_ControlPoints.GetSize()>1)
	{
		POINT pPoint1 = (POINT)m_ControlPoints.GetAt(0);
		POINT pPoint2 = (POINT)m_ControlPoints.GetAt(1);
		POINT pPoint3 = (POINT)m_ControlPoints.GetAt(2);

		pPoint1.y = -(pPoint1.y);
		pPoint2.y = -(pPoint2.y);
		pPoint3.y = -(pPoint3.y);

		m_ControlPoints.RemoveAll();

		AddControlPoint(pPoint1);
		AddControlPoint(pPoint2);
		AddControlPoint(pPoint3);
	}
}

/*---------------------------------------------------------------------------------
	Function to mirror entity with respect to y-axis
---------------------------------------------------------------------------------*/
void CGeoPolybezier::MirrorAboutYaxis()
{
		/* Negate the Ycoordinate of the start and end points, and then the
	   control points */
	m_StPnt.x = -(m_StPnt.x);	
	m_EndPnt.x = -(m_EndPnt.x);

	/*Time to negate the control points' Ycoordinate*/
	if(m_ControlPoints.GetSize()>1)
	{
		POINT pPoint1 = (POINT)m_ControlPoints.GetAt(0);
		POINT pPoint2 = (POINT)m_ControlPoints.GetAt(1);
		POINT pPoint3 = (POINT)m_ControlPoints.GetAt(2);

		pPoint1.x = -(pPoint1.x);
		pPoint2.x = -(pPoint2.x);
		pPoint3.x = -(pPoint3.x);

		m_ControlPoints.RemoveAll();

		AddControlPoint(pPoint1);
		AddControlPoint(pPoint2);
		AddControlPoint(pPoint3);
	}
}


/*---------------------------------------------------------------------------------
	Function to clone the entity. This function returns a new pointer to
	cloned entity.
---------------------------------------------------------------------------------*/
CPrimitive* CGeoPolybezier::CreateClone()
{
	CGeoPolybezier* pClone = new CGeoPolybezier(*this);
	return pClone;
}