/*
 *  By: Sunil Kumar
 *  GeoIO Module. Module to draw simple 2D geometry objects and serialization.
 *  This code is free and is distributed in the hope that it will be useful for simple learning and experimental purpose only WITHOUT any WARRANTY.
 *  The author shall hold no redistribution liability in case of any unethical usage/modification.
 *  Distributed under CPOL.
 */
// LayerList.h: interface for the CLayerList class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __LAYERLIST_H__
#define __LAYERLIST_H__

#include "stdafx.h"
#include "Layer.h"

class GEOIO_EXPORT CLayerList  :  public CObject
{
	DECLARE_SERIAL(CLayerList)
public:
	CLayerList();
	CLayerList(CLayerList& pLayerList);
	virtual ~CLayerList();

	CPrimitive* GetEntityForRedoOperationInActiveLayer();
	int GetEntityCountInUndoList();
	int GetEntityCountInActiveLayer();
	void AddEntityToUndoList(CPrimitive *pEntity);
	CPrimitive* GetEntityForUndoOperationInActiveLayer();
	void DrawAllLayers(CDC *pDC);
	BOOL RemoveLayerAtIndex(int nIndex);
	void MakeAllLayersInactive();
	void DrawActiveLayer(CDC *pDC);
	void DrawAllVisibleLayers(CDC *pDC);
	void AddEntityToActiveLayer(CPrimitive *pEntity);
	int GetLayersCount();
	void DeleteAllLayers();
	void AddLayerToList(CLayer *pLayer);	
	void Serialize(CArchive &ar);

	CLayer* GetActiveLayer();
	CLayer* GetLayerAtIndex(int nIndex);
	CLayer* GetPreviousLayer();
	CLayer* GetNextLayer();


private:
	int m_nLayerNumber;
	CTypedPtrList<CObList, CLayer*> m_LayerList;

};
#endif // __LAYERLIST_H__
