/*
 *  By: Sunil Kumar
 *  GeoIO Module. Module to draw simple 2D geometry objects and serialization.
 *  This code is free and is distributed in the hope that it will be useful for simple learning and experimental purpose only WITHOUT any WARRANTY.
 *  The author shall hold no redistribution liability in case of any unethical usage/modification.
 *  Distributed under CPOL.
 */
// Primitive.h: interface for the CPrimitive class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __PRIMITIVE_H__
#define __PRIMITIVE_H__

#include "stdafx.h"

class GEOIO_EXPORT CPrimitive : public CObject  
{
	
public:	
	CPrimitive();
	virtual ~CPrimitive();

public:
	BOOL m_bEntityDeleted;
	CPoint m_StPnt;
	CPoint m_EndPnt;
	COLORREF m_clr;
	int m_nEntityNum;
	GEO_ENTITY_TYPE m_geoEntityType;

public:
	virtual void SetStartPoint(CPoint startPnt);
	virtual CPoint GetStartPoint();
	virtual void SetEndPoint(CPoint endPnt);
	virtual CPoint GetEndPoint();
	virtual BOOL IsEntityDeleted();
	virtual void SetDeleted();
	virtual void SetEntityType(GEO_ENTITY_TYPE geoType);
	virtual GEO_ENTITY_TYPE GetEntityType();
	virtual COLORREF GetEntityColor();
	virtual void SetEntityColor(COLORREF clr);	
	virtual void SetEntityNumber(int nEntityNum);
	virtual int GetEntityNumber();

	virtual void Draw(CDC *pDC) = 0;

	virtual void DeleteEntity() = 0;


	virtual BOOL IsPointNearEntity(CPoint refPnt) = 0;

	// function to mirror the entity about x-axis
	virtual void MirrorAboutXaxis(void)= 0;

	// function to mirror the entity about y-axis
	virtual void MirrorAboutYaxis(void)= 0;

	virtual CPrimitive* CreateClone(void) = 0;
};
#endif // __PRIMITIVE_H__